using System;
using System.Collections.Generic;
using System.Text;
using System.Data.OleDb;
using System.Data;
using System.Data.Common;

namespace Utils.DataBase
{
    /// <summary>
    /// Manager of SQL query in an ACCESS environment
    /// </summary>
    public sealed class OleDbSql : DbSql
    {
        #region Variables
        /// <summary>
        /// Path of the database
        /// </summary>
        private string vDBPath = "";
        /// <summary>
        /// Represents a connection to the database
        /// </summary>
        private OleDbConnection oConn = new OleDbConnection();
        /// <summary>
        /// Represents an SQL statement or stored procedure to execute against a data
        /// source. Provides a base class for database-specific classes that represent
        /// commands.
        /// </summary>
        private OleDbCommand oCom;
        /// <summary>
        /// Aids implementation of the System.Data.IDbDataAdapter interface. Inheritors
        /// of System.Data.Common.DbDataAdapter implement a set of functions to provide
        /// strong typing, but inherit most of the functionality needed to fully implement
        /// a DataAdapter.
        /// </summary>
        private OleDbDataAdapter oDa;
        /// <summary>
        /// Indicates if the stored procedures is ready to be called
        /// </summary>
        private bool vIsPreparedProcedure = false;
        /// <summary>
        /// Indicates the error which occured at the opening of the connection
        /// </summary>
        private Exception vOpenError;
        /// <summary>
        /// Indicates if the call of the procedure or the SQL query is managed that means that at TRUE
        /// the object manage the connection (it opens and closes automatically) at FALSE the user has to open
        /// and close connection manualy
        /// </summary>
        private bool vIsManagedPrepared = true;
        #endregion

        #region Constructeur
        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="_Path">Emplacement de la base de donnes</param>
        public OleDbSql(string _Path)
        {
            vConnectionString = "Provider=Microsoft.Jet.OLEDB.4.0;Data Source=\"";
            DbPath = _Path;
        }
        #endregion

        #region Mthodes
        /// <summary>
        /// Returns the value of the last ID inserted in the Database
        /// </summary>
        /// <returns>Object which contains the las primary key</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        protected override object LastID()
        {
            try
            {
                vSqlQuery = "SELECT @@IDENTITY";
                //oConn = new OleDbConnection(vFullString);
                oCom = new OleDbCommand(vSqlQuery, oConn);
                oCom.CommandType = CommandType.Text;

                //oConn.Open();
                object LastKey = oCom.ExecuteScalar();
                //oConn.Close();
                return LastKey;
            }
            catch (OleDbException) { throw; }
            catch (InvalidOperationException) { throw; }
        }
        /// <summary>
        /// Execute a Selection query
        /// </summary>
        /// <param name="_SqlQuery">The Selection SQL query</param>
        /// <param name="_ds">DataSet wich is filled by the query</param>
        /// <param name="_IsManagedConnection">If TRUE, the connection is automatically opened and closed. If FALSE, use manually Open() and Close()</param>
        /// <returns>A DataSet which contains the data of the SELECT</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        public override int ExecuteSql(string _SqlQuery, ref DataSet _ds, bool _IsManagedConnection)
        {
            try
            {
                vSqlQuery = _SqlQuery;

                if (_IsManagedConnection)
                    oConn = new OleDbConnection(vFullString);

                oCom = new OleDbCommand(vSqlQuery, oConn);
                oCom.CommandType = CommandType.Text;

                oDa = new OleDbDataAdapter(oCom);

                int nbr = oDa.Fill(_ds);
                return nbr;
            }
            catch (OleDbException) { throw; }
            catch (InvalidOperationException) { throw; }
        }
        /// <summary>
        /// Execute a Selection query
        /// </summary>
        /// <param name="_SqlQuery">The Selection SQL query</param>
        /// <param name="_ds">DataSet wich is filled by the query</param>
        /// <returns>A DataSet which contains the data of the SELECT</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        public override int ExecuteSql(string _SqlQuery, ref DataSet _ds) { return ExecuteSql(_SqlQuery, ref _ds, true); }
        /// <summary>
        /// Execute a Selection query
        /// </summary>
        /// <param name="_ds">DataSet wich is filled by the query</param>
        /// <returns>A DataSet which contains the data of the SELECT</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        public override int ExecuteSql(ref DataSet _ds)
        {
            if (vSqlQuery == "")
                throw new ArgumentException("The SQL query is empty!", "vSqlQuery");
            return ExecuteSql(vSqlQuery, ref _ds, true);
        }

        /// <summary>
        /// Execute a query which does return only one value (SELECT MAX, SELECT COUNT, ...)
        /// </summary>
        /// <param name="_SqlQuery">The SQL query</param>
        /// <param name="_IsManagedConnection">If TRUE, the connection is automatically opened and closed. If FALSE, use manually Open() and Close()</param>
        /// <returns>The sole value of the query</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        public override object ExecuteScalar(string _SqlQuery, bool _IsManagedConnection)
        {
            try
            {
                vSqlQuery = _SqlQuery;
                if (_IsManagedConnection)
                    oConn = new OleDbConnection(vFullString);
                oCom = new OleDbCommand(vSqlQuery, oConn);
                oCom.CommandType = CommandType.Text;

                if (_IsManagedConnection)
                    oConn.Open();

                object LastKey = oCom.ExecuteScalar();

                if (_IsManagedConnection)
                    oConn.Close();
                return LastKey;
            }
            catch (OleDbException) { throw; }
            catch (InvalidOperationException) { throw; }
        }
        /// <summary>
        /// Execute a query which does return only one value (SELECT MAX, SELECT COUNT, ...)
        /// </summary>
        /// <param name="_SqlQuery">La requt SQL</param>
        /// <returns>The sole value of the query</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        public override object ExecuteScalar(string _SqlQuery) { return ExecuteScalar(_SqlQuery, true); }
        /// <summary>
        /// Execute a query which does return only one value (SELECT MAX, SELECT COUNT, ...)
        /// </summary>
        /// <returns>The sole value of the query</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        public override object ExecuteScalar()
        {
            if (vSqlQuery == "")
                throw new ArgumentException("The SQL query is empty!", "vSqlQuery");
            return ExecuteScalar(vSqlQuery, true);
        }

        /// <summary>
        /// Execute a SQL command wich does not receive arguments (INSERT, DELETE, UPDATE)
        /// </summary>
        /// <param name="_SqlQuery">Sql query</param>
        /// <param name="_IsManagedConnection">If TRUE, the connection is automatically opened and closed. If FALSE, use manually Open() and Close()</param>
        /// <returns>Return the number of rows that the command modified</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        public override int ExecuteNonQuery(string _SqlQuery, bool _IsManagedConnection)
        {
            try
            {
                vSqlQuery = _SqlQuery;
                if (_IsManagedConnection)
                    oConn = new OleDbConnection(vFullString);
                oCom = new OleDbCommand(vSqlQuery, oConn);
                oCom.CommandType = CommandType.Text;

                if (_IsManagedConnection)
                    oConn.Open();

                int i = oCom.ExecuteNonQuery();

                if (_IsManagedConnection)
                    oConn.Close();

                return i;
            }
            catch (OleDbException) { throw; }
            catch (InvalidOperationException) { throw; }
        }
        /// <summary>
        /// Execute a SQL command wich does not receive arguments (INSERT, DELETE, UPDATE)
        /// </summary>
        /// <param name="_SqlQuery">The SQL query</param>
        /// <returns>The number of rows the Query modified</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        public override int ExecuteNonQuery(string _SqlQuery) { return ExecuteNonQuery(_SqlQuery, true); }
        /// <summary>
        /// Execute a SQL command wich does not receive arguments (INSERT, DELETE, UPDATE)
        /// </summary>
        /// <returns>Le nombre de tuples impliqu dans la commande SQL</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        public override int ExecuteNonQuery()
        {
            if (vSqlQuery == "")
                throw new ArgumentException("The SQL query is empty!", "vSqlQuery");
            return ExecuteNonQuery(vSqlQuery, true);
        }

        /// <summary>
        /// Prepare the use of a stored procedure
        /// </summary>
        /// <param name="_ProcedureName">Procedure name</param>
        /// <param name="_IsManagedConnection">If TRUE, the connection is automatically opened and closed. If FALSE, use manually Open() and Close()</param>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="ArgumentException"> The value was not a valid System.Data.OleDb.OleDbCommand.CommandType.</exception>
        public override void PrepareProcedure(string _ProcedureName, bool _IsManagedConnection)
        {
            vIsManagedPrepared = _IsManagedConnection;
            try
            {
                vSqlQuery = _ProcedureName;
                if (_IsManagedConnection)
                    oConn = new OleDbConnection(vFullString);

                oCom = new OleDbCommand(vSqlQuery, oConn);
                oCom.CommandType = CommandType.StoredProcedure;
                vIsPreparedProcedure = true;
            }
            catch (OleDbException) { throw; }
            catch (ArgumentException) { throw; }

        }
        /// <summary>
        /// Prepare the use of a stored procedure
        /// </summary>
        /// <param name="_ProcedureName">Procedure name</param>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="ArgumentException"> The value was not a valid System.Data.OleDb.OleDbCommand.CommandType.</exception>
        public override void PrepareProcedure(string _ProcedureName) { PrepareProcedure(_ProcedureName, true); }

        /// <summary>
        /// Execute the Selection Procedure prepared with PrepareProcedure
        /// </summary>
        /// <param name="_ds">DataSet qui sera rempli par le rsultat de la requte</param>
        /// <returns>The number of rows that the query modified</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        /// <exception cref="ProcedureNonPreparedException">Si on tente d'excut la procdure alors qu'elle n'est pas prpare</exception>
        public override int ExecuteSqlProcedure(ref DataSet _ds)
        {
            if (!vIsPreparedProcedure) throw new ProcedureNonPreparedException();
            try
            {
                if (vIsManagedPrepared)
                    oDa = new OleDbDataAdapter(oCom);

                int nbr = oDa.Fill(_ds);
                oCom.Parameters.Clear();
                return nbr;
            }
            catch (OleDbException) { throw; }
            catch (InvalidOperationException) { throw; }
        }
        /// <summary>
        /// Execute the Scalar Stored Procedure prepared with PrepareProcedure
        /// </summary>
        /// <returns>The sole value of the query</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        /// <exception cref="ProcedureNonPreparedException">Si on tente d'excut la procdure alors qu'elle n'est pas prpare</exception>
        public override object ExecuteScalarProcedure()
        {
            if (!vIsPreparedProcedure) throw new ProcedureNonPreparedException();
            try
            {
                if (vIsManagedPrepared)
                    oConn.Open();
                object LastKey = oCom.ExecuteScalar();

                if (vIsManagedPrepared)
                    oConn.Close();
                oCom.Parameters.Clear();
                return LastKey;
            }
            catch (OleDbException) { throw; }
            catch (InvalidOperationException) { throw; }
        }
        /// <summary>
        /// Execute a Stored Procedure which does not return results (INSERT, UPDATE).
        /// </summary>
        /// <remarks>Please prepare this method by calling PrepareProcedure before</remarks>
        /// <returns>The number of rows that the query modified</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        /// <exception cref="ProcedureNonPreparedException">Si on tente d'excut la procdure alors qu'elle n'est pas prpare</exception>
        public override int ExecuteNonQueryProcedure()
        {
            if (!vIsPreparedProcedure) throw new ProcedureNonPreparedException();
            try
            {
                if (vIsManagedPrepared)
                    oConn.Open();
                int i = oCom.ExecuteNonQuery();

                if (vIsManagedPrepared)
                    oConn.Close();
                oCom.Parameters.Clear();
                return i;
            }
            catch (OleDbException) { throw; }
            catch (InvalidOperationException) { throw; }
        }
        /// <summary>
        /// Execute a Stored Procedure which does not return results (INSERT, UPDATE).
        /// </summary>
        /// <remarks>Please prepare this method by calling PrepareProcedure before</remarks>
        /// <param name="_LastID">Last modified ID</param>
        /// <returns>The number of rows that the query modified</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        /// <exception cref="ProcedureNonPreparedException">Si on tente d'excut la procdure alors qu'elle n'est pas prpare</exception>
        public override int ExecuteNonQueryProcedure(ref object _LastID)
        {
            if (!vIsPreparedProcedure) throw new ProcedureNonPreparedException();
            try
            {
                if (vIsManagedPrepared)
                    oConn.Open();
                int i = oCom.ExecuteNonQuery();
                _LastID = this.LastID();

                if (vIsManagedPrepared)
                    oConn.Close();
                oCom.Parameters.Clear();
                return i;
            }
            catch (OleDbException) { throw; }
            catch (InvalidOperationException) { throw; }
        }

        /// <summary>
        /// Open a connection to the database
        /// </summary>
        public override bool Open()
        {
            if (vFullString != "")
            {
                try
                {
                    oConn = new OleDbConnection(vFullString);
                    oConn.Open();
                    return true;
                }
                catch (Exception ex)
                {
                    vOpenError = ex;
                    return false;
                }
            }
            return false;
        }
        /// <summary>
        /// Close the running connection
        /// </summary>
        public override void Close()
        {
            if (oConn.State != ConnectionState.Closed)
            {
                oConn.Close();
            }
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Gets or sets the database path
        /// </summary>
        public string DbPath
        {
            get { return vDBPath; }
            set
            {
                vDBPath = value;
                vFullString = vConnectionString + vDBPath + "\"";
            }
        }
        /// <summary>
        /// Gets or Sets the parameters of the stored procedure
        /// </summary>
        public new OleDbParameterCollection Parameters { get { return oCom.Parameters; } }
        /// <summary>
        /// Gets the state of the Database connection
        /// </summary>
        public override ConnectionState ConnectionState { get { return oConn.State; } }
        /// <summary>
        /// Gets the exception which was thrown or NULL if none exception was thrown. Once the exception was readed, it's replaced by NULL
        /// </summary>
        public override Exception OpenError
        {
            get
            {
                Exception buf = vOpenError;
                vOpenError = null;
                return buf;
            }
        }
        /// <summary>
        /// Gets or sets the SQL query of this instance
        /// </summary>
        public override string SqlQuery { get { return vSqlQuery; } set { vSqlQuery = value; } }
        #endregion
    }
}
