using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.IO;

namespace Utils.Misc
{
    /// <summary>
    /// Root class for options
    /// </summary>
    [Serializable]
    public class OptionContainer : Serializer
    {
        #region Variables
        /// <summary>
        /// Referece on itself for the Singletton Pattern
        /// </summary>
        private static OptionContainer refOptions = null;
        /// <summary>
        /// Generic object which contains the options
        /// </summary>
        private object vOptions = null;
        /// <summary>
        /// Indicates whether the options was already affected
        /// </summary>
        private bool vAffected = false;
        #endregion

        #region Constructors
        /// <summary>
        /// Constructor
        /// </summary>
        protected OptionContainer() : base(ApplicationPath.CommandPath + "\\Options.srz") { }
        /// <summary>
        /// Gets the reference on the OptionContainer object
        /// </summary>
        public static OptionContainer Instance
        {
            get
            {
                if (refOptions == null)
                {
                    try
                    { refOptions = (OptionContainer)Deserialize(ApplicationPath.CommandPath + "\\Options.srz"); }
                    catch (FileNotFoundException) { refOptions = new OptionContainer(); }
                }
                return refOptions;
            }
        }
        /// <summary>
        /// Process before destruction
        /// </summary>
        ~OptionContainer() { base.Serialize(this); }
        #endregion

        #region Properties
        /// <summary>
        /// Gets or sets the generic object which contains the options
        /// </summary>
        public object Options
        {
            get { return vOptions; }
            set
            {
                vAffected = true;
                vOptions = value;
            }
        }
        /// <summary>
        /// Gets the values wich indicates whether the Options was affected
        /// </summary>
        public bool Affected { get { return vAffected; } }
        #endregion
    }
}
